@@ -288,7 +288,7 @@ class Code:
                 doc = self.get_docstring_from_code(module, function, filepath, classname)
                 if doc is None:
                     doc = ""
-                found.set('Description', doc)
+                found.set('Shape_description', doc)
                 if classname is not None:
                     if len(classname) == 0:
                         del found.attrib["Class"]
@@ -464,7 +464,7 @@ class Code:
             output_variable = shape.get("@Output_variable")
             if output_variable == variable:
                 return module, classname
-        return None
+        return None, None
 
     class TempDict:
 
@@ -504,7 +504,7 @@ class Code:
         if len(function) == 0:
             title = f"Create {classname} object"
         newentry = {'xml': '', 'w': 120, 'h': 80, 'aspect': 'fixed', 'title': title}
-        newshape = f"<mxGraphModel><root><mxCell id=\"0\"/><mxCell id=\"1\" parent=\"0\"/><object label=\"{title}\" Module=\"{module}\" Class=\"{classname}\" Function=\"{function}\" Output_variable=\"\" id=\"2\" Description=\"\"><mxCell style=\"shape=ext;rounded=1;html=1;whiteSpace=wrap;\" vertex=\"1\" parent=\"1\"><mxGeometry width=\"120\" height=\"80\" as=\"geometry\"/></mxCell></object></root></mxGraphModel>"
+        newshape = f"<mxGraphModel><root><mxCell id=\"0\"/><mxCell id=\"1\" parent=\"0\"/><object label=\"{title}\" Module=\"{module}\" Class=\"{classname}\" Function=\"{function}\" Output_variable=\"\" id=\"2\" Shape_description=\"\"><mxCell style=\"shape=ext;rounded=1;html=1;whiteSpace=wrap;\" vertex=\"1\" parent=\"1\"><mxGeometry width=\"120\" height=\"80\" as=\"geometry\"/></mxCell></object></root></mxGraphModel>"
         root = eltree.ElementTree(eltree.fromstring(newshape))
         found = root.find('.//root/object')
         if module is None:
@@ -541,6 +541,8 @@ class Code:
                     if key.lower() != "self":
                         if str(value).__contains__("="):
                             val = str(value).split("=")[1].replace("\'", "").strip()
+                            if val.lower() == "none":
+                                val = ""
                         else:
                             if str(value).lower().__contains__("bool"):
                                 val = False
@@ -562,7 +564,7 @@ class Code:
                         found.set("label", title)
                 if len(function) == 0:
                     del found.attrib["Function"]
-                found.set("Description", doc)
+                found.set("Shape_description", doc)
                 if len(classname) == 0 and len(variable) == 0:
                     del found.attrib["Class"]
                 if len(variable) > 0:
@@ -593,6 +595,8 @@ class Code:
         Add (or update) descriptions from code to all shapes in a library.
         :param filepath: The full path to the library file.
         """
+        if filepath == r"d:\runtime\prd\BPMN\flows\Monitor_24x7_Exchange.xml":
+            print("")
         dct, original = self.openflow(filepath)
         graph = dct.get("mxGraphModel")
         root = graph.get("root")
@@ -600,33 +604,26 @@ class Code:
         try:
             if steps is not None:
                 for shape in steps:
-                    if hasattr(shape,"@Class"):
-                        classname = shape.get("@Class")
-                    else:
-                        classname = None
-                    if hasattr(shape, "@Module"):
-                        module = shape.get("@Module")
-                    else:
-                        module = None
-                    if hasattr(shape, "@Function"):
-                        function = shape.get("@Function")
-                    else:
-                        function = None
-                    if hasattr(shape, "@Output_variable"):
-                        variable = shape.get("@Output_variable")
-                    else:
-                        variable = None
-                    if hasattr(shape, "@label"):
-                        label = shape.get("@label")
-                    else:
-                        label = None
+                    shapetype = shape.get("@Type")
+                    classname = shape.get("@Class")
+                    module = shape.get("@Module")
+                    function = shape.get("@Function")
+                    variable = shape.get("@Output_variable")
+                    label = shape.get("@label")
                     if classname is not None:
                         if classname.startswith("%") and classname.endswith("%"):
                             module, classname = self.search_modulename_in_flow(classname, steps)
-                    doc = self.get_docstring_from_code(module, function, filepath, classname)
+                    if function is not None or classname is not None or module is not None:
+                        doc = self.get_docstring_from_code(module, function, filepath, classname)
+                    else:
+                        doc = None
                     if doc is None:
                         doc = ""
-                    shape.update({'@Description': doc})
+                    shape.update({'@Shape_description': doc})
+                    # try:
+                    #     shape.pop("@Description")
+                    # except:
+                    #     pass
                     if classname is not None:
                         if len(classname) == 0:
                             shape.pop("@Class")
@@ -636,7 +633,7 @@ class Code:
                     # found.set('tooltip', doc)
             self.saveflow(filepath, dct, original)
         except (ValueError, Exception):
-            raise Exception("Error. Probably the file is not a BPMN-RPA flow or the flow is incorrect.")
+            raise Exception(f"Error. Probably the file {filepath} is not a BPMN-RPA flow or the flow is incorrect.")
 
     @staticmethod
     def get_functions_from_module(module: str) -> Any:
@@ -708,8 +705,8 @@ class Code:
 
 
 # c = Code()
-# c.libpath = r"C:\PythonProjects\BPMN_RPA\BPMN_RPA\Shapes.xml"
-# c.sort_library(r"C:\PythonProjects\BPMN_RPA\BPMN_RPA\Shapes.xml")
+# c.libpath = r"C:\Temp\Libraries\Shapes.xml"
+# c.sort_library(c.libpath)
 # c.run_flow(r"c:\temp\msgbox.xml", "hey!")
 # c.add_shape_from_function_to_library(module=r"C:\PythonProjects\BPMN_RPA\BPMN_RPA\Scripts\Code.py",
 #                                       function="get_docstring_from_code", title="Get comments from Python code",
