@@ -1280,22 +1280,26 @@ class Visio:
         count = 1
         while True:
             if f"visio/pages/page{count}.xml" in self.root:
-                connects = self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Connects").get("Connect")
-                if connects:
-                    for shp in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Connects").get("Connect"):
+                for shp in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Connects").get("Connect"):
+                    blnupdate = False
+                    if [x for x in retn if x["@ID"] == shp["@FromSheet"]]:
+                        conn = [x for x in retn if x["@ID"] == shp["@FromSheet"]][0]
+                        blnupdate = True
+                    else:
                         conn = {"@ID": shp["@FromSheet"], "type": "connector"}
-                        if shp["@FromCell"] == "BeginX":
-                            obj = [x for x in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Shapes").get("Shape") if x["@ID"] == shp["@FromSheet"]][0]
-                            text = None
-                            if "Text" in obj:
-                                text = obj["Text"]
-                            if text is not None:
-                                conn.update({"value": text})
-                            conn.update({"source": shp["@ToSheet"]})
-                            target = [y for y in [x for x in connects if x["@FromSheet"] == shp["@FromSheet"]] if y["@FromCell"] == "EndX"][0]["@ToSheet"]
-                            conn.update({"target": target})
-                            retn.append(conn)
-                    count += 1
+                        obj = [x for x in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Shapes").get("Shape") if x["@ID"] == shp["@FromSheet"]][0]
+                        text = None
+                        if "Text" in obj:
+                            text = obj["Text"]
+                        if text is not None:
+                            conn.update({"value": text})
+                    if shp["@FromCell"] == "EndX":
+                        conn.update({"target": shp["@ToSheet"]})
+                    else:
+                        conn.update({"source": shp["@ToSheet"]})
+                    if not blnupdate:
+                        retn.append(conn)
+                count += 1
             else:
                 break
         return retn
@@ -1354,49 +1358,30 @@ class Visio:
         :param shape: The Shape-object
         :returns: A Step-object
         """
-        properties = {"id": shape.get("@ID")}
-        properties.update({"type": "shape"})
-        if "@Name" in shape:
-            if "Text" in shape:
-                properties.update({"name": shape["Text"]})
-            else:
-                properties.update({"name": shape["@Name"]})
-        properties.update({"IsStart": False})
         retn = self.dynamic_object()
-        if "type" in shape:
-            if str(shape["type"]).lower().__contains__("connector"):
-                properties.pop("IsStart")
-                properties.update({"type": "connector"})
-                properties.update({"target": shape["target"]})
-                properties.update({"source": shape["source"]})
-                if "value" in shape:
-                    properties.update({"value": shape["value"]})
-                    properties.update({"name": shape["value"]})
-                for k, v in properties.items():
-                    setattr(retn, k, v)
-                return retn
-
+        retn.id = shape.get("@ID")
+        retn.type = "Shape"
+        retn.IsStart = False
         # Get default attributes if any
+        master = None
         standard_attributes = []
-        props = []
         target_id = None
-        # region attributes from template
         if "@Master" in shape:
             master = shape["@Master"]
-            obj = [x for x in self.root[f"visio/masters/masters.xml"].get("Masters").get("Master") if x["@ID"] == master]
+            obj = [x for x in self.root[f"visio/masters/masters.xml"].get("Masters").get("Master") if x["@ID"]==master]
             if obj:
                 r_id = obj[0]["Rel"]["@r:id"]
                 obj = [x for x in self.root[f"visio/masters/_rels/masters.xml.rels"].get("Relationships").get("Relationship") if x["@Id"] == r_id]
                 if obj:
                     target_id = obj[0]["@Target"]
-                    obj = [x for x in self.root[f"visio/masters/{target_id}"]["MasterContents"]["Shapes"]["Shape"]["Section"] if x["@N"] == "Property"]
+                    obj = [x for x in self.root[f"visio/masters/{target_id}"]["MasterContents"]["Shapes"]["Shape"]["Section"] if x["@N"]=="Property"]
                     if obj:
                         if "Cell" in obj[0]["Row"]:
-                            standard_attributes = obj[0]["Row"]["Cell"]
+                            standard_attributes= obj[0]["Row"]["Cell"]
                         else:
                             for rw in obj[0]["Row"]:
-                                for attr in rw["Cell"]:
-                                    standard_attributes.append(attr)
+                                    for attr in rw["Cell"]:
+                                        standard_attributes.append(attr)
         key = None
         value = None
         for rw in standard_attributes:
@@ -1405,86 +1390,90 @@ class Visio:
             if rw["@N"] == "Value":
                 value = rw["@V"]
             if key is not None and value is not None:
-                properties.update({"row_id": "template", str(key).lower(): value})
+                setattr(retn, str(key).replace("@", "").lower(), value)
                 # reset key and value
                 key, value = None, None
-        labels = []
         if target_id is not None:
-            obj = [x for x in self.root[f"visio/masters/{target_id}"]["MasterContents"]["Shapes"]["Shape"]["Section"] if x["@N"] == "Property"]
-            if obj:
-                labels = obj[0]["Row"]
-        # endregion
+            labels = [x for x in self.root[f"visio/masters/{target_id}"]["MasterContents"]["Shapes"]["Shape"]["Section"] if x["@N"] == "Property"][0]["Row"]
+        else:
+            labels = []
         if "Section" in shape:
             if not isinstance(shape["Section"], list):
-                sect = [shape["Section"]]
+                col = [shape["Section"]]
             else:
-                sect = [x for x in shape["Section"] if x["@N"] == "Property"]
-            # sect is a list or ordered dict. Get the 'Property' section
-            if [x for x in sect if x["@N"] == "Property"]:
-                props = [x for x in sect if x["@N"] == "Property"][0].get("Row")
-            # props is now the 'Property' section of the Shape
-            # Loop the properties
-            # props is ordered_dict based on stencil, but list[ordered_dict] for manual
-            if not isinstance(props, list):
-                props = [props]
-            for prop in props:
-                # get the property row_id
+                col = [x for x in shape["Section"] if x["@N"] == "Property"]
+            if [x for x in col if x["@N"] == "Property"]:
+                col = [x for x in col if x["@N"] == "Property"][0].get("Row")
                 row_id = None
-                if str(prop.get("@N")).lower().__contains__("row_"):
-                    row_id = prop.get("@N")
-                # region get the key and value
-                key, value = None, None
-                if "Cell" not in prop:
-                    continue
-                if isinstance(prop.get("Cell"), dict):
-                    # it is a value that belongs to a template label
-                    label = [x for x in labels if x["@N"] == row_id]
-                    kvps = None
-                    if label:
-                        kvps = label[0]["Cell"]
-                    rw_k = [x for x in kvps if x["@N"] == "Label"]
-                    if rw_k:
-                        key = rw_k[0]["@V"]
-                    value = prop.get("Cell")["@V"]
-                else:
-                    rw_k = [x for x in prop.get("Cell") if x["@N"] == "Label"]
-                    if rw_k:
-                        key = rw_k[0]["@V"]
-                    rw_l = [x for x in prop.get("Cell") if x["@N"] == "Value"]
-                    if rw_l:
-                        value = rw_l[0]["@V"]
-                    # endregion
-                if key is not None and value is not None:
-                    properties.update({"row_id": row_id, str(key).lower(): value})
+                if labels:
+                    if isinstance(labels, list):
 
-        if "Section" in shape:
-            for rw in shape["Section"]:
-                if str(rw).lower().__contains__("('@v', 'exclusive gateway')") and str(rw).lower().__contains__("('@n', 'type')"):
-                    properties.update({"type": "exclusive gateway"})
-                    properties.update({"name": ""})
-                if str(rw).lower().__contains__("('@v', 'parallel gateway')") and str(rw).lower().__contains__("('@n', 'type')"):
-                    properties.update({"type": "parallel gateway"})
-                    properties.update({"name": ""})
+                            if "Cell" in col:
+                                row_id = col["@N"]
+                                col = col["Cell"]
+                            if not isinstance(col, list):
+                                test = [x for x in labels if x["@N"] == row_id]
+                                if test:
+                                    label = [y for y in [x for x in labels if x["@N"] == row_id][0]["Cell"] if y["@N"] == "Label"][0]["@V"]
+                                key = label
+                                value = col["@V"]
+                                setattr(retn, str(key).replace("@", "").lower(), value)
+                                # reset key and value
+                                key, value = None, None
+                            else:
+                                for rw in col:
+                                    row_id = rw["@N"]
+                                    if "Cell" in rw:
+                                        rw = rw["Cell"]
+                                    test = [x for x in labels if x["@N"] == row_id]
+                                    if test:
+                                        label = [y for y in [x for x in labels if x["@N"] == row_id][0]["Cell"] if y["@N"] == "Label"][0]["@V"]
+                                    key = label
+                                    if rw["@N"] == "Value":
+                                        value = rw["@V"]
+                                    if key is not None and value is not None:
+                                        if str(key).lower() == "class":
+                                            key = "classname"  # 'class' is a reserved keyword, so use 'classname'
+                                        if str(key).lower() != "name":
+                                            setattr(retn, str(key).replace("@", "").lower(), value)
+                                            # reset key and value
+                                            key, value = None, None
+                                        continue
+            else:
+                for att in shape:
+                    setattr(retn, str(att).replace("@", "").lower(), shape[att])
+        else:
+            for att in shape:
+                setattr(retn, str(att).replace("@", "").lower(), shape[att])
 
+        if str(shape.get("@NameU")).__contains__("connector"):
+            retn.type = "connector"
+            if not shape["Text"] is None:
+                retn.value = shape["Text"]
+                retn.name = shape["Text"]
+        else:
+            if "Section" in shape:
+                for rw in shape["Section"]:
+                    if str(rw).lower().__contains__("('@v', 'exclusive gateway')") and str(rw).lower().__contains__("('@n', 'type')"):
+                        retn.type = "exclusive gateway"
+                        retn.name = ""
+                    if str(rw).lower().__contains__("('@v', 'parallel gateway')") and str(rw).lower().__contains__("('@n', 'type')"):
+                        retn.type = "parallel gateway"
+                        retn.name = ""
         if "type" not in shape:
             count = 1
             while True:
                 if f"visio/pages/page{count}.xml" in self.root:
                     if not [x for x in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Connects").get("Connect") if x["@FromCell"] == "EndX" and x["@ToSheet"] == shape["@ID"]]:
-                        properties.update({"IsStart": True})
-                        properties.update({"name": "Start"})
-                        properties.update({"type": "shape"})
+                        retn.IsStart = True
+                        retn.name = "Start"
+                        retn.type = "shape"
                     if not [x for x in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Connects").get("Connect") if x["@FromCell"] == "BeginX" and x["@ToSheet"] == shape["@ID"]]:
-                        properties.update({"name": "End"})
-                        properties.update({"type": "shape"})
+                        retn.name = "End"
+                        retn.type = "shape"
                     count += 1
                 else:
                     break
-        if str(properties["name"]).lower().__contains__("gateway"):
-            properties.update({"name": ""})
-        properties.update({"type": str(properties["type"]).lower()})
-        for k, v in properties.items():
-            setattr(retn, k, v)
         return retn
 
 # Test
